@@ -1,5 +1,5 @@
 # Tibshirani, Walther and Hastie (2000) #
-# Check added 02JUL04                   #
+# Check added 02JUL04;correction to uniform sampling 15MAY06    #
 gap<-function(data=swiss,class=g,B=500){
 library(stats)
 if (min(table(class))==1) stop("Singleton clusters not allowed")
@@ -13,7 +13,8 @@ z1 <- matrix(data = NA, nrow = nrow(x1), ncol = ncol(x1))
   tots <- 0
 for (k in 1:B){
    for (j in 1:ncol(x1)){
-         z1[,j] <- x1[sample(1:nrow(x1),nrow(x1),replace=TRUE),j]
+         min.x <- min(x1[,j]);max.x <- max(x1[,j])
+         z1[,j] <- runif(length(nrow(x1)), min = min.x, max = max.x) # x1[sample(1:nrow(x1),nrow(x1),replace=TRUE),j]
       }
 z <- crossprod(t(z1),t(veigen))
 tots<-tots+log(sum(by(crossprod(t(z),veigen),factor(class),intern <- function(x) sum(dist(x)/ncol(x))/2)))/B
