@@ -1,7 +1,8 @@
-# samrocNboot, author Per Broberg, version 31AUG02, modified 13NOV02, 27MAY04  #
-samrocNboot <- function (data = M, formula = ~as.factor(g), contrast = c(0,1), N = c(50, 100, 200, 300), B = 100, perc = 0.6, smooth = FALSE, w = 1, measure = "euclid") 
+# samrocNboot, author Per Broberg, version 31AUG02, modified 13NOV02, 27MAY04, 31MAY06  #
+samrocNboot <- function (data = M, formula = ~as.factor(g), contrast = c(0,1), N = c(50, 100, 200, 300), B = 100, perc = 0.6, smooth = FALSE, w = 1, measure = "euclid", probeset = NULL) 
 {
     data <- as.matrix(data)
+    nrows <- nrow(data);ncols = ncol(data)
 #  data <- utmat;formula <- ~as.factor(g);contrast <- c(0, 1);N <- c(50,100,200,300)
 #  B <- 100;perc <- 0.6;smooth <- F;measure <- c("euclid");w <- 1
     utest <- Xprep(indata = data, formula = formula, contrast = contrast)
@@ -17,11 +18,11 @@ samrocNboot <- function (data = M, formula = ~as.factor(g), contrast = c(0,1), N
     ssq <- quantile(ss, probs = seq(0, perc, by = 0.05))
     ssq <- c(0, ssq)
     ss <- sqrt(utest$Vest/utest$k)
-    diffs <- matrix(nrow = nrow(as.matrix(data)), ncol = B)
+    diffs <- matrix(nrow = nrows, ncol = B)
     diffs1<-t(diffs)
-    sses <- matrix(nrow = nrow(as.matrix(data)), ncol = B)
-    varses <- matrix(nrow = nrow(as.matrix(data)), ncol = B)
-    pj <- numeric(length = nrow(as.matrix(data)))
+    sses <- matrix(nrow = nrows, ncol = B)
+    varses <- matrix(nrow = nrows, ncol = B)
+    pj <- numeric(length = nrows)
     pts <- matrix(nrow = length(ssq), ncol = length(N))
     p.is <- matrix(nrow = length(ssq), ncol = length(N))
     target <- numeric(length(ssq))
@@ -65,8 +66,9 @@ samrocNboot <- function (data = M, formula = ~as.factor(g), contrast = c(0,1), N
 #        pj <- t(crossprod(c(rep(1/ncol(test), ncol(test))), t(test > 0)))
 #        pj <- apply(as.matrix(di), 1, function(x) mean(abs(dstari) >  abs(x)))
         pt <- numeric(length(N))
-        p.i <- apply(as.matrix(alpha), 1, function(x) max(pj[pj <  x]))
-        p.is[i, ] <- p.i
+#        p.i <- apply(as.matrix(alpha), 1, function(x) max(pj[pj <  x]))
+#        p.is[i, ] <- p.i
+        p.is[i, ] <- alpha
         for (j in 1:length(alpha)) pt[j] <- mean((pj < alpha[j]))
            pts[i, ] <- pt
     }
@@ -103,7 +105,26 @@ samrocNboot <- function (data = M, formula = ~as.factor(g), contrast = c(0,1), N
         fp <- p0 * pj
         fn <- 1 - p0 * (1 - pj) - rank(pj)/length(pj)
         errors <- fp + fn
-    list(d = di,diffs=diffs, diff = utest$Mbar, se = sqrt(utest$Vest/utest$k), 
-        d0 = dstari, p0 = p0, s0 = snew, pvalues = pj, N = N.opt, 
-        errors=errors)
+    if(is.null(probeset)) probeset <- paste("probeset",1:nrows)
+res <-   new("samroc.result", 
+             d = as.vector(di), 
+             diff = as.vector(utest$Mbar),
+             se = as.vector(sqrt(utest$Vest/utest$k)),
+             d0 = dstari,
+             p0 = p0,
+             s0 = snew,
+             pvalues = pj,
+             N.list = as.integer(N.opt),
+             errors = errors,
+             formula = formula,
+             contrast = contrast,
+             annotation = as.character(date()),
+             N.sample = ncols,
+             B = as.integer(B),
+             call = as.character(match.call()),
+             id = as.character(probeset),
+             error.df = as.integer(utest$f)
+)
+
+res 
 }
